

require_relative 'service_catalog'

module Column_ID
    Unit = 0
    Load = 1
    Active = 2
    Sub = 3
    Descr = 4
    Native_info = 5
    State = 6
end

class FilteredServiceListStore < Gtk::TreeModelFilter
	attr_accessor :filterstr
end

class ServiceListStore < Gtk::ListStore
	include Column_ID
	def self.new_all_service
		sl = ServiceListStore.new
		sl.load_catalog
		sl
	end
	def initialize
		super(String, String, String, String, String, String, String)
	end
  def load_catalog
    sci = ServiceCatalog.instance
    puts "Loading Catalog with #{sci.size} item(s)"
    sci.each_value{|s| append_service(s) }
  end
  def reload
    clear  # clear Gtk::ListStore
    ServiceCatalog.instance.rebuild
    load_catalog
  end
	def append_managed_service(name)
		if service =  ServiceCatalog.instance[name] 
			append_service(service)
		end
	end
	def append_service(service)
		row = self.append 
		row[ Unit ] = service
        row[ Load ] = service.load
        row[ Active ] = service.active
        row[ Sub ] = service.sub
		row[ Descr  ] = service.descr
        row[ Native_info ] = service.native_info
        row[ State ] = service.state
	end	
  def sort_by_unit
    self.set_sort_column_id( Unit, :ascending)
  end    
end

class TreeViewColumnToggle < Gtk::TreeViewColumn
	@@cell_renderer =  Gtk::CellRendererToggle.new
    def initialize(title, column_id )
        attrs = { :active => column_id }
        super(title, @@cell_renderer , attrs )
        self.resizable = true 
        self.sort_column_id = column_id
    end
end

class TreeViewColumnText < Gtk::TreeViewColumn
	@@cell_renderer =  Gtk::CellRendererText.new
    def initialize(title, column_id )
        attrs = { :text => column_id }
        super(title, @@cell_renderer , attrs )
        self.resizable = true 
        self.sort_column_id = column_id
    end
end

class ServiceListView < Gtk::TreeView
  include Column_ID
	def initialize(list)
		super(list)
		@list = list
		@col1 = TreeViewColumnText.new("Unit", Unit )
    
		@col1.fixed_width = 200
		@col1.sort_indicator = true
        
		@col2 = TreeViewColumnText.new("Load", Load )
		@col3 = TreeViewColumnText.new("Active", Active )        
		@col4 = TreeViewColumnText.new("Sub", Sub)  
		@col5 = TreeViewColumnText.new("Description", Descr)
		@col6 = TreeViewColumnText.new("Native?", Native_info)
		@col7 = TreeViewColumnText.new("State", State)   

        
    self.append_column(@col1)
    self.append_column(@col6)
    self.append_column(@col7)
  	self.append_column(@col2)
    self.append_column(@col3)
  	self.append_column(@col4)
    self.append_column(@col5)
        
    self.enable_search = true
	end

	def signal_connect_sel_changed(sctrl)
		@sel = self.selection
		@sel.mode = :browse
		@sel.signal_connect("changed"){
		  if @sel.selected
        @service = ServiceCatalog.instance[@sel.selected[Unit] ]
			  sctrl.service = @service
			  sctrl.execute("Status")
			else
			  sctrl.init_textv
			end
		}
	end
    
  def update_selected_row
    iter = self.model.convert_iter_to_child_iter( @sel.selected )
	iter.set_value(Load, @service.load)
    iter.set_value(Active, @service.active)
    iter.set_value(Sub, @service.sub)
  end 
end
