require_relative 'index' 
require 'pp'

class SystemService < String
   include Indexable
   attr_accessor :descr
   attr_accessor :state
   attr_accessor :load
   attr_accessor :active
   attr_accessor :sub
   attr_accessor :overrides_legacy
   attr_accessor :filename_full
   
   @@cmd_info = "sudo systemctl status" 
   @@descr_rgx = /\A.+\.service\s+-\s+([[:print:]]+)\Z/
   @@descr_masked_rgx = /\A.+\.service\Z/   
   @@loaded_rgx = /\ALoaded:\s+([[:graph:]]+)\s+\((.*)\)\Z/
   @@active_rgx = /\AActive:\s+([[:graph:]]+)\s+\(([[:graph:]]+)\).*\Z/
   @@warning_rgx = /\AWarning:/
   @@status_read_failed_rgx = Regexp.new('\AFailed to get properties:.*\Z')
   @@status_unit_notfound_rgx = Regexp.new('\AUnit\s[[:graph:]]+\scould\snot\sbe\sfound.\Z')   
   def set_status_info_from_output( action_output )
        @status_info_raw = action_output
        self.parse_status_info
   end
# This provides the texts to index (module Indexable)    
   	def each_sentence
		yield self.to_str
		yield @descr unless  @descr.nil?
	end
   def parse_status_info
        match = nil
		return if self.handle_masked_info
        info_tab = @status_info_raw.lines.reject{|l| @@warning_rgx.match(l) }
        info_tab.map!{|r| r.strip}
        if @@status_read_failed_rgx.match(info_tab[0])
            return nil
        end    
       if @@status_unit_notfound_rgx.match(info_tab[0])
            return nil
        end                
        if match = @@descr_rgx.match(info_tab[0])
            self.descr = match[1]
        else
# masked script/service without description returned     
            if match = @@descr_masked_rgx.match(info_tab[0])
            else
                puts info_tab[0]
                raise "Failed parsing line printed above"
            end

        end 
# complete data
        if match
            match = @@loaded_rgx.match(info_tab[1])
            if match 
                self.load = match[1]  
                load_info = match[2].split(';')
                self.filename_full = load_info[0]
                self.state = load_info[1]
            end    
            match = @@active_rgx.match(info_tab[2].strip)
            if match
                self.active = match[1]   
                self.sub = match[2]
            end
        end

   end
   def handle_masked_info
        if self.state == 'masked'
            @load = 'masked'
            @active = 'inactive'
            @sub = 'dead'
            true
        else
			false
        end   
   end
   def read_status_info
        return nil if self.end_with?("@")
        return if self.handle_masked_info
		@status_info_raw, status = Open3.capture2("#@@cmd_info #{self.to_s}" )
		if @status_info_raw.lines.size > 0
            self.parse_status_info
        else
# TODO: a more decent error handling...        
             puts "Failed getting service status info for"
             pp self
        end
   end
   def read_state
        return nil if self.end_with?("@")
        match = nil
		stdout_str, status = Open3.capture2("#@@cmd_info #{self.to_s}" )
		if stdout_str.lines.size > 0

            info_tab = stdout_str.lines.reject{|l| @@warning_rgx.match(l) }
            if @@status_read_failed_rgx.match(info_tab[0].strip)
                return nil
            end            
            match = @@loaded_rgx.match(info_tab[1].strip)
            if match 
                load_info = match[2].split(';')
                self.filename_full = load_info[0]
                self.state = load_info[1]
            end    
                

        else
             puts "Failed getting service state info for"
             pp self
              
        end
   end
# default constructor, takes unit name   
   def initialize(str)
      super(str)
      @overrides_legacy = false
   end
   def native_info
# yes+   : native and overrides init script
# yes    : native and no init script
# no     : init script   
     if    self.native?
        @overrides_legacy ? "yes+" : "yes"
     else
		"no"
     end
   end
   def native?
      true
   end
    
end

class SystemServiceLegacy < SystemService

   def initialize(str)
        super(str)        
        self.read_status_info
   end
   def native?
        false
   end
end

